/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2014 Tiago Cerqueira
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Tiago Cerqueira <1090678@isep.ipp.pt> <tiago.miguel43@gmail.com>
 */

#include "ns3/routes-mobility-helper.h"

NS_LOG_COMPONENT_DEFINE ("RoutesMobilityModel");

namespace ns3 {

RoutesMobilityHelper::RoutesMobilityHelper ()
{
  NS_LOG_FUNCTION (this);
  m_strategy = new GoogleMapsApiConnect (); //TODO:Inquire about creating the strategy
  m_travelMode = "driving";
  m_departureTime = GetCurrentTime ();
}

RoutesMobilityHelper::RoutesMobilityHelper (double lat, double lng, double altitude)
{
  NS_LOG_FUNCTION (this);
  m_strategy = new GoogleMapsApiConnect (lat, lng,altitude);
  m_travelMode = "driving";
  m_departureTime = GetCurrentTime ();
}

RoutesMobilityHelper::RoutesMobilityHelper (const RoutesMobilityHelper& orig)
{
  this->m_strategy = orig.m_strategy;
  this->m_travelMode = orig.m_travelMode;
}

RoutesMobilityHelper::~RoutesMobilityHelper ()
{
  NS_LOG_FUNCTION (this);
  free (m_strategy);
}
std::string
RoutesMobilityHelper::GetCurrentTime ()
{
  NS_LOG_FUNCTION (this);
  std::ostringstream strs;
  std::time_t time = std::time (NULL);
  std::asctime (std::localtime (&time));
  strs << time;
  return strs.str ();
}
int
RoutesMobilityHelper::SchedulePoints (Ptr<Node> node, std::list<Ptr<Leg> > legList)
{
  NS_LOG_FUNCTION (this);
  int addedWaypoints = 0;
  std::list<Ptr<Step> > stepList;
  std::list<Ptr<Point> > pointList;
  Ptr<Point> p;
  bool first = false;
  Ptr<WaypointMobilityModel> waypointMobility = DynamicCast<WaypointMobilityModel> ( node->GetObject<MobilityModel> ());
  for (std::list<Ptr<Leg> >::iterator i = legList.begin (); i != legList.end (); i++)
    {
      (*i)->GetStepList (stepList);
      for (std::list<Ptr<Step> >::iterator j = stepList.begin (); j != stepList.end (); j++)
        {
          NS_LOG_DEBUG ("Iterating step " << *(*j));
          pointList = (*j)->GetPointList ();
          for (std::list<Ptr<Point> >::iterator k = pointList.begin (); k != pointList.end (); k++)
            {
              p = ((*k));
              if (first)
                {
                  p->SetWaypointTime (((p->GetWaypointTime ()) + 0.001));
                  first = false;
                }
              waypointMobility->AddWaypoint (Waypoint (Seconds (p->GetWaypointTime ()),Vector (p->GetXCoordinate (),p->GetYCoordinate (),p->GetZCoordinate ())));
              NS_LOG_DEBUG (*p << "Waypoint number " << (addedWaypoints + 1) << " added to Node " << node->GetId () << std::endl << "---------------------");
              addedWaypoints++;
            }
          first = true;
        }
    }
  NS_ASSERT_MSG (addedWaypoints != 0, "No waypoints were added");
  NS_LOG_INFO ("Added " << addedWaypoints << " waypoints to Node " << node->GetId ());
  return addedWaypoints;
}

int
RoutesMobilityHelper::ChooseRoute (std::string startPoint, std::string endPoint, Ptr<Node> node)
{
  NS_LOG_FUNCTION (this);
  std::list<Ptr<Leg> > legList;
  legList = m_strategy->PerformRequest (startPoint,endPoint, m_travelMode, m_departureTime);
  if (g_log.IsEnabled (LOG_DEBUG))
    {
      uint32_t i = 0;
      for (std::list<Ptr<Leg> >::iterator list_iter = legList.begin ();
           list_iter != legList.end(); list_iter++, i++)
        {
          NS_LOG_DEBUG ("Leg " << i << ": " << *list_iter);
        }
    }
  SchedulePoints (node,legList);
  return 0;
}
//listTokenStartEndPoint is supposed to be filled with pairs of start and endpoints
int
RoutesMobilityHelper::ChooseRoute (std::list<std::string> const &listTokenStartEndPoint, NodeContainer &nodeContainer)
{
  NS_LOG_FUNCTION (this);
  int sizeOfContainer = nodeContainer.GetN ();
  NS_ASSERT_MSG ( (listTokenStartEndPoint.size () / 2) == nodeContainer.GetN (), "The number of nodes and the number of routes do not match!");
  std::string start = "",end = "";
  Ptr<Node> node;
  std::list<std::string>::const_iterator j = listTokenStartEndPoint.begin ();
  for (int i = 0; i < sizeOfContainer; i++,j++)
    {
      node = nodeContainer.Get (i);
      start = (*j);
      j++;
      end = (*j);
      ChooseRoute (start,end,node);
    }
  return 0;
}

int
RoutesMobilityHelper::ChooseRoute (std::string const *listTokenStartEndPoint, NodeContainer &nodeContainer)
{
  NS_LOG_FUNCTION (this);
  int sizeOfContainer = nodeContainer.GetN ();
  std::string start = "", end = "";
  Ptr<Node> node;
  for (int i = 0, j = 0; i < sizeOfContainer; i++, j++)
    {
      node = nodeContainer.Get (i);
      start = listTokenStartEndPoint[j];
      j++;
      end = listTokenStartEndPoint[j];
      ChooseRoute (start, end, node);
    }
  return 0;
}
int
RoutesMobilityHelper::ChooseRoute (Ptr<Node> node, std::string path)
{
  NS_LOG_FUNCTION (this);
  std::list<Ptr<Leg> > legList;
  legList = m_strategy->PerformOfflineRequest (path);
  SchedulePoints (node,legList);
  return 0;
}

int
RoutesMobilityHelper::ChooseRoute (NodeContainer &nodeContainer, std::list<double> latLngTokens, double radius)
{
  NS_LOG_FUNCTION (this);
  int i = 0, k = 1, size = nodeContainer.GetN (), numOfUniqueRoutes = 0;
  double lat = 0, lng = 0;
  numOfUniqueRoutes = (((latLngTokens.size ())) * 15);
  NS_ASSERT_MSG (size <= numOfUniqueRoutes, "The number of nodes is superior to the number of locations that can be obtained from the Google Maps Places API. Please add more locations or remove nodes from the node container.");
  for (std::list<double>::iterator j = latLngTokens.begin (); j != latLngTokens.end () && i < size; j++,i++,k++)
    {
      NodeContainer auxNodeContainer;
      lat = (*j);
      j++;
      lng = (*j);
      for (; i < (k * 30) && i < size; i++)
        {
          auxNodeContainer.Add (nodeContainer.Get (i));
        }
      ChooseRoute (auxNodeContainer, lat, lng, radius);
    }
  return 0;
}

int
RoutesMobilityHelper::ChooseRoute (NodeContainer &nodeContainer, double lat, double lng, double radius)
{
  NS_LOG_FUNCTION (this);
  std::ostringstream strs;
  std::string start = "",endpoint = "";
  GoogleMapsPlacesApiConnect places;
  std::list<Ptr<Place> > placeList;
  Ptr<Place> place;
  unsigned int j = 0;
  placeList = places.PerformRequest (lat,lng,radius);
  for (std::list<Ptr<Place> >::iterator i = placeList.begin (); i != placeList.end () && j < nodeContainer.GetN (); i++,j++)
    {
      strs.str ("");
      strs.clear ();
      strs << (*i)->GetLatitude ();
      strs << ",";
      strs << (*i)->GetLongitude ();
      start = strs.str ();
      i++;
      strs.str ("");
      strs.clear ();
      strs << (*i)->GetLatitude ();
      strs << ",";
      strs << (*i)->GetLongitude ();
      endpoint = strs.str ();
      ChooseRoute (start, endpoint, nodeContainer.Get (j));
    }
  return 0;
}

int
RoutesMobilityHelper::ChooseRoute (NodeContainer &nodeContainer, std::string dirPath)
{
  NS_LOG_FUNCTION (this);
  std::list<std::string> files;
  files = Open (dirPath);
  int nodeContainerSize = nodeContainer.GetN ();
  NS_ASSERT_MSG (files.size () == nodeContainer.GetN (), "The number of nodes is not equal to the number of XML files ");
  int j = 0;
  std::string absolutePath = "";
  for (std::list<std::string>::iterator i = files.begin (); i != files.end () && j < nodeContainerSize; i++, j++)
    {
      absolutePath = dirPath + "/" + (*i);
      ChooseRoute (nodeContainer.Get (j), absolutePath);
    }
  return 0;
}
std::list<std::string>
RoutesMobilityHelper::Open (std::string dirPath)
{
  NS_LOG_FUNCTION (this);
  DIR* dir;
  dirent* pdir;
  std::list<std::string> files;

  dir = opendir (dirPath.c_str ());

  while ((pdir = readdir (dir)))
    {
      if (!((strcmp (pdir->d_name, ".") == 0) || (strcmp (pdir->d_name,"..") == 0)))
        {
          files.push_back (pdir->d_name);
        }
    }

  return files;
}
int RoutesMobilityHelper::ChooseRoute (Ptr<Node> node, std::string start, std::string end, std::string redirectedDestination, double timeToTrigger)
{
  NS_LOG_FUNCTION (this);
  std::list<Ptr<Step> > stepList;
  std::list<Ptr<Point> > pointList;
  Time lastTime;
  Ptr<Point> p, pt;
  bool isRedirected = false;
  std::ostringstream strs;
  std::list<Ptr<Leg> > legList, auxLegList;
  double lastPointTime;
  legList = m_strategy->PerformRequest (start, end, m_travelMode, m_departureTime);
  for (std::list<Ptr<Leg> >::iterator i = legList.begin (); i != legList.end (); i++)
    {
      (*i)->GetStepList (stepList);
      for (std::list<Ptr<Step> >::iterator j = stepList.begin (); j != stepList.end () && !isRedirected; j++)
        {
          pointList = (*j)->GetPointList ();
          for (std::list<Ptr<Point> >::iterator k = pointList.begin (); k != pointList.end () && !isRedirected; k++)
            {
              if ((*k)->GetWaypointTime () > timeToTrigger)
                {
                  p = ((*k));
                  lastTime = Time (p->GetWaypointTime ());
                  pointList.erase (k, pointList.end ());
                  strs.str ("");
                  strs.clear ();
                  strs << p->GetLatitude ();
                  strs << ",";
                  strs << p->GetLongitude ();
                  auxLegList = m_strategy->PerformRequest (strs.str (), redirectedDestination, m_travelMode, m_departureTime);
                  isRedirected = true;
                }
            }
          if (isRedirected)
            {
              stepList.erase (j,stepList.end ());
            }
        }
      (*i)->SetStepList (stepList);
    }
  for (std::list<Ptr<Leg> >::iterator i = auxLegList.begin (); i != auxLegList.end (); i++)
    {
      (*i)->GetStepList (stepList);
      for (std::list<Ptr<Step> >::iterator j = stepList.begin (); j != stepList.end (); j++)
        {
          pointList = (*j)->GetPointList ();
          for (std::list<Ptr<Point> >::iterator k = pointList.begin (); k != pointList.end (); k++)
            {
              pt = ((*k));
              lastPointTime = lastTime.GetSeconds ();
              lastPointTime += ((p->GetWaypointTime () + pt->GetWaypointTime ()) + 0.001);
              pt->SetWaypointTime (lastPointTime);
            }
        }
    }
  for (std::list<Ptr<Leg> >::iterator i = auxLegList.begin (); i != auxLegList.end (); i++)
    {
      legList.push_back ((*i));
    }
  SchedulePoints (node, legList);
  return 0;
}

void
RoutesMobilityHelper::SetTransportationMethod (std::string travelMode)
{
  NS_LOG_FUNCTION (this);
  this->m_travelMode = travelMode;
}
void
RoutesMobilityHelper::SetDepartureTime (std::string departureTime)
{
  NS_LOG_FUNCTION (this);
  this->m_departureTime = departureTime;
}
int
RoutesMobilityHelper::DownloadDirectionsXml (std::string startPoint, std::string endPoint, std::string pathToFilename)
{
  NS_LOG_FUNCTION (this);
  m_strategy->PerformRequest (startPoint, endPoint, m_travelMode, m_departureTime, true, pathToFilename);
  return 0;
}
int
RoutesMobilityHelper::DownloadDirectionsXml (std::list<std::string> listTokenStartEndPoint, std::string pathToFolder)
{
  NS_LOG_FUNCTION (this);
  std::string pathToFile = "";
  std::string start = "", end = "";
  for (std::list<std::string>::iterator i = listTokenStartEndPoint.begin (); i != listTokenStartEndPoint.end (); i++)
    {
      start = (*i);
      i++;
      end = (*i);
      pathToFile = pathToFolder;
      pathToFile.append ("/" + start + " - " + end + ".xml");
      DownloadDirectionsXml (start, end, pathToFile);
      pathToFile = "";
    }
  return 0;
}
} //namespace ns3
