/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2014 Tiago Cerqueira
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Tiago Cerqueira <1090678@isep.ipp.pt> <tiago.miguel43@gmail.com>
 */

#include <GeographicLib/Geocentric.hpp>
#include <GeographicLib/LocalCartesian.hpp>
#include <list>
#include "ns3/simulator.h"
#include "ns3/boolean.h"
#include "ns3/config.h"
#include "ns3/routes-mobility-helper.h"
#include "ns3/test.h"

/**
 * \brief Tests the RoutesMobilityModel conversion capabilities
 * 
 * This test performs comparisons between latitude and longitude values obtained from Google's Polyline Encoder Utility and values obtained from the conversion algorithms implemented in the module
 * (Google's Polyline enconder utility - https://developers.google.com/maps/documentation/utilities/polylineutility )
 * 
 * It also tests the Cartesian coordinates obtained from the GeographicLib library conversion functions.
 * 
 */

namespace ns3 {

class RoutesMobilityModelTest : public TestCase
{
public:
  RoutesMobilityModelTest () : TestCase ("Check RoutesMobilityModel conversion capabilities")
  {
    m_gmapsStrategy = new GoogleMapsApiConnect (41.1621429, -8.6218531,0);
  }
  virtual ~RoutesMobilityModelTest ()
  {
  }

private:
  GoogleMapsApiConnect* m_gmapsStrategy;
  virtual void DoRun (void);
};

void
RoutesMobilityModelTest::DoRun (void)
{
  Ptr<Leg> leg;
  Ptr<Step> step;
  std::list<Ptr<Leg> > legList;
  std::list<Ptr<Step> > stepList;
  std::list<Ptr<Point> > pointList, comparePointList;
  legList = m_gmapsStrategy->PerformOfflineRequest ("src/mobility/test/routes-mobility-model-xml-test.xml");

  //Create the expected values
  Ptr<Point> p1,p2,p3,p4,p5,p6;
  p1 = new Point (41.18063,-8.609350000000001);
  p2 = new Point (41.17956, -8.60930000000001);
  p3 = new Point (41.17938, -8.60930000000001);
  p4 = new Point (41.178790000000006, -8.609250000000001);
  p5 = new Point (41.17817, -8.60923);
  p6 = new Point (41.17734, -8.60918);

  p1->SetXCoordinate (1049.07);
  p1->SetYCoordinate (2053.2);
  p1->SetZCoordinate (-0.417406);

  p2->SetXCoordinate (1053.29);
  p2->SetYCoordinate (1934.37);
  p2->SetZCoordinate (-0.380866);

  p3->SetXCoordinate (1053.29);
  p3->SetYCoordinate (1914.38);
  p3->SetZCoordinate (-0.374821);

  p4->SetXCoordinate (1057.49);
  p4->SetYCoordinate (1848.86);
  p4->SetZCoordinate (-0.35614);

  p5->SetXCoordinate (1059.18);
  p5->SetYCoordinate (1780.0);
  p5->SetZCoordinate (-0.336785);

  p6->SetXCoordinate (1063.39);
  p6->SetYCoordinate (1687.82);
  p6->SetZCoordinate (-0.312367);

  p1->SetWaypointTime (0.0);
  p2->SetWaypointTime (13.3308);
  p3->SetWaypointTime (15.572);
  p4->SetWaypointTime (22.9331);
  p5->SetWaypointTime (30.655);
  p6->SetWaypointTime (41.0);


  //Create the point list
  pointList.push_back (p1);
  pointList.push_back (p2);
  pointList.push_back (p3);
  pointList.push_back (p4);
  pointList.push_back (p5);
  pointList.push_back (p6);


  leg = (legList.front ());
  leg->GetStepList (stepList);
  step = stepList.front ();
  comparePointList = step->GetPointList ();

  for (std::list<Ptr<Point> >::iterator i = comparePointList.begin (),j = pointList.begin (); i != comparePointList.end (); i++,j++)
    {
      NS_TEST_ASSERT_MSG_EQ_TOL ((*i)->GetLatitude (), (*j)->GetLatitude (), 0.1, "The points do not match!");
      NS_TEST_ASSERT_MSG_EQ_TOL ((*i)->GetLongitude (), (*j)->GetLongitude (), 0.1, "The points do not match!");
      NS_TEST_ASSERT_MSG_EQ_TOL ((*i)->GetXCoordinate (), (*j)->GetXCoordinate (), 0.1, "The points do not match!");
      NS_TEST_ASSERT_MSG_EQ_TOL ((*i)->GetYCoordinate (), (*j)->GetYCoordinate (), 0.1, "The points do not match!");
      NS_TEST_ASSERT_MSG_EQ_TOL ((*i)->GetZCoordinate (), (*j)->GetZCoordinate (), 0.1, "The points do not match!");
      NS_TEST_ASSERT_MSG_EQ_TOL ((*i)->GetWaypointTime (), (*j)->GetWaypointTime (), 0.1, "The points do not match!");
    }

  Simulator::Stop (Seconds (42));
  Simulator::Run ();
  Simulator::Destroy ();
}

static struct RoutesMobilityModelTestSuite : public TestSuite
{
  RoutesMobilityModelTestSuite () : TestSuite ("routes-mobility-model", UNIT)
  {
    AddTestCase (new RoutesMobilityModelTest (), TestCase::QUICK);
  }
} g_routesMobilityModelTestSuite;

} // namespace ns3

